 /*
  RTK GPS Test software for two SparFun GPS-RTK2(ZED-F9P) modules
  ************************
  File: RTK_Azimuth2d.ino
  ************************
  SHT75 Sensiron Temp & Humidity Sensor
  Bosch BME280 sensor via I2C (rel. Humidity, Pressure)
  //DS18B20 temp sensor via 1-wire
  SM6AFV 2020-04-03  
  Rev. 1.02
  SparkFun GPS-RTK Lib 1.7.2
  Arduino Ide 1.8.5
  Teensyduino 1.49
  Teensy 4.0 Processor(600 MHz)
*/
#include <SPI.h>
#include <ILI9341_t3.h>
#include <Wire.h> //Needed for I2C to GPS
#include "SparkFun_Ublox_Arduino_Library.h"
#include <SHT1X.h>                 // Sensirion Adrduino library
#include <Adafruit_Sensor.h>       // uncomment for BME280 Sensor
#include <Adafruit_BME280.h>       // uncomment for BME280 Sensor
//#include <OneWire.h>             // uncomment for DS18B20 Sensor
//#include <DallasTemperature.h>   // uncomment for DS18B20 Sensor


//DEFINE ONE-WIRE PIN
//#define ONE_WIRE_BUS 9           // uncomment for DS18B20 one-wire Temp. Sensor

#include <font_Arial.h>                     // High Res. Font
#include <font_ArialBold.h>                 // High Res. Font

// SWITCH PINS
#define SW1 3   // Azimuth
#define SW2 4   // Weather
#define SW3 5   // GPS Data

// SHT75 PINS (1=sck, 2=Vdd, 3=GND, 4=data)
#define dataSHT 7     // (4)
#define clockSHT 9    // (1)

// TEENSY SPI PINS FOR TFT DISPLAY
#define TFT_DC 20
#define TFT_CS 21
#define TFT_RST 8
#define TFT_MOSI 11
#define TFT_SCLK 13
#define TFT_MISO 12

// OBJECTS
ILI9341_t3 tft = ILI9341_t3(TFT_CS, TFT_DC, TFT_RST, TFT_MOSI, TFT_SCLK, TFT_MISO);
SFE_UBLOX_GPS myGPS;  // Connect to ROVER GPS Unit via I2C
SHT1x sht1x(dataSHT, clockSHT);
Adafruit_BME280 bme;  // I2C bus
//OneWire oneWire(ONE_WIRE_BUS);
//DallasTemperature sensors(&oneWire);

long lastTime1 = 0;  // GPS Data
long LastTime2 = 0;  // Weather Data i.e. BME280
long LastTime3 = 0;  // Temp i.e. DS18B20

// GLOBAL VARIABLES
int year;
int month;
int day;
int hour;
int minute;
int second;
double latitude;
double longitude;
double haccuracy;
double vaccuracy;
double altitude;
double heading;
double relposlength;
int fixedtype;
int siv;
boolean gnssfixok;
boolean diffsoln;
boolean relposvalid;
int carrsoln;
boolean ismoving;
boolean refposmiss;
boolean refobsmiss;
double temperature1 = 0;
double temperature2 = 0;
double rhumidity = 0;
double dewpoint1 = 0;
double dewpoint2 = 0;
double pressure = 0;
double tempSHT = 0;
double humiditySHT = 0;
double dewpointSHT = 0;
double loclong = 0;
double loclat = 0;

String MHlocator;
double freq = 47.0;
double absHumidity = 0.;
double GasLoss = 0;

// ADC Input definitions 
const int batteryPin1 = A1;
const int batteryPin2 = A2;
const float referenceVoltage = 3.28;
const int adcMax = 4095;
const float R1 = 10000;
const float R2 = 1000;
float resistorFactor = (R1+R2)/R2;
// parameters for battery check
const float BatteryVoltageMin = 10.5;
int adVoltage1 = 0;
int adVoltage2 = 0;
float BatteryVoltage = 0.0;

//-------------------------------------------------------------------------------
void setup()
{
  // PIN MODES
  pinMode(SW1, INPUT_PULLUP);  // Azimuth Display
  pinMode(SW2, INPUT_PULLUP);  // Weather Display
  pinMode(SW3, INPUT_PULLUP);  // GPS Disply

  // ANALOG INPUT
  pinMode(batteryPin1, INPUT);
  pinMode(batteryPin2, INPUT);
  //analogReference(INTERNAL);
  analogReadRes(12); 
  analogReadAveraging(19);
  
  Serial.begin(115200);
  initDisplay();
  Wire.begin();
  Wire.setClock(460800); //I2C clock speed

  if (myGPS.begin() == false) //Connect to the Ublox module using Wire I2C port
  {
    Serial.println(F("Ublox GPS not detected at default I2C address. Please check wiring. Freezing."));
    tft.println("Ublox GPS not detected at default I2C address (0x42). Check wiring!");
    //while (1);
  }
  if (!bme.begin(0x76, &Wire)) 
  {
  Serial.println("Could not find a valid BME280 sensor, check wiring!");
  tft.println("Could not find a valid BME280 sensor. Check wiring!");
  //while (1);
  }
  //Sensor.begin();
}
//--------------------------------------------------------------
void loop()
{
  get_GPS_Data();
  getWeather();
  AbsoluteHumidity();
  getMHlocator();

  if (digitalRead(SW1) == LOW)
  {
    showAzimuth();
    getBatteryStatus();
  }
  if (digitalRead(SW2) == LOW)showWeather();
  if (digitalRead(SW3) == LOW)show_GPS_Data();
}
//--------------------------------------------------------------
void get_GPS_Data()
{
  if (millis() - lastTime1 > 1000)
  {
    lastTime1 = millis();
    if (myGPS.getRELPOSNED() == true)
    {
      //Date_Time
      year = myGPS.getYear();
      month = myGPS.getMonth();
      day = myGPS.getDay();
      hour = myGPS.getHour();
      minute = myGPS.getMinute();
      second = myGPS.getSecond();

      //Navigation Data
      latitude = float(myGPS.getHighResLatitude()) / 10000000.;
      longitude = float(myGPS.getHighResLongitude()) / 10000000.;
      haccuracy = float(myGPS.getHorizontalAccuracy()) / 10;
      vaccuracy = float(myGPS.getVerticalAccuracy()) / 10;
      altitude = float(myGPS.getMeanSeaLevel()) / 1000.;
      heading = float((myGPS.relPosInfo.relPosHeading) / 100000.);
      relposlength = float(myGPS.relPosInfo.relPosLength) * 10.;
      fixedtype = myGPS.getFixType();
      siv = myGPS.getSIV();
      gnssfixok = myGPS.relPosInfo.gnssFixOk;
      diffsoln = myGPS.relPosInfo.diffSoln;
      relposvalid = myGPS.relPosInfo.relPosValid;
      carrsoln = myGPS.relPosInfo.carrSoln;
      ismoving = myGPS.relPosInfo.isMoving;
      refposmiss = myGPS.relPosInfo.refPosMiss;
    }
  }
}
//--------------------------------------------------------------

void show_GPS_Data()
{
  tft.fillScreen(ILI9341_BLACK);
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(0, 10);
  tft.setFont(Arial_10_Bold);
  tft.println("RTK-GPS NAVIGATION DATA:");
  tft.println("-----------------------------------------------");
  tft.setTextColor(ILI9341_WHITE);
  tft.setFont(Arial_10);
  char buffer[15];

  //Date_Time
  tft.print("UTC DateTime  ");
  tft.print(year);
  if (month <10) tft.print("-0");
  else tft.print("-");
  tft.print(month);
  if (day <10) tft.print("-0");
  else tft.print("-");
  tft.print(day);
  tft.print(" ");
  if (hour <10) tft.print("0");
  else tft.print(" ");
  tft.print(hour);
  if (minute <10) tft.print("0");
  else tft.print(":");
  tft.print(minute);
  if (second <10) tft.print(":0");
  else tft.print(":");
  tft.println(second);
  tft.println();

  //Navigation Data
  tft.print("HP Latitude     (deg):   ");
  dtostrf(latitude, 13, 9, buffer);
  tft.println(buffer);

  tft.print("HP Longitude  (deg):   ");
  dtostrf(longitude, 13, 9, buffer);
  tft.println(buffer);

  tft.print("Hor. Accuracy  (mm):");
  dtostrf(haccuracy, 10, 1, buffer);
  tft.println(buffer);

  tft.print("Vert. Accuracy(mm):  ");
  dtostrf(vaccuracy, 9, 1, buffer);
  tft.println(buffer);

  tft.print("AltitudeMSL        (m):");
  dtostrf(altitude, 12, 2, buffer);
  tft.println(buffer);

  tft.print("Ant. Heading   (deg): ");
  dtostrf(heading, 10, 2, buffer);
  tft.println(buffer);

  tft.print("relPosLength   (mm):   ");
  dtostrf(relposlength, 10, 2, buffer);
  tft.println(buffer);

  tft.print("FixType:                          ");
  if (fixedtype == 0)
    tft.println("-");
  if (fixedtype == 3)
    tft.println("3D");
  if (fixedtype == 4)
    tft.println("GNSS");
  tft.println();

  tft.print("SatFix:                            ");
  tft.println(siv);

  tft.print("gnssFixOk:                      ");
  if (gnssfixok == true)
    tft.println("x");
  else
    tft.println("-");

  tft.print("diffSolution:                     ");
  if (diffsoln == true)
    tft.println("x");
  else
    tft.println("-");

  tft.print("relPosValid:                     ");
  if (relposvalid == true)
    tft.println("x");
  else
    tft.println("-");

  tft.print("carrier Solution Type:     ");
  if (carrsoln == 0)
    tft.println("None");
  else if (carrsoln == 1)
    tft.println("Float");
  else if (carrsoln == 2)
    tft.println("Fixed");

  tft.print("isMoving:                         ");
  if (ismoving == true)
    tft.println("x");
  else
    tft.println("-");

  tft.print("refPosMiss:                     ");
  if (refposmiss == true)
    tft.println("x");
  else
    tft.println("-");

  tft.print("refObsMiss:                     ");
  if (refobsmiss == true)
    tft.println("x");
  else
    tft.println("-");

  tft.print("MHLocator:                  ");
  tft.println(MHlocator);
  //delay(2000);
}
//--------------------------------------------------------------
void initDisplay()
{
  // Start TFT
  tft.begin();
  tft.setRotation(4); // Orientation (2 or 4 Potrait)
  tft.fillScreen(ILI9341_BLACK);
  tft.setCursor(0, 80);
  tft.setFont(Arial_10_Bold);
  tft.print(" GPS Direction/Weather Indicator");
  tft.setCursor(90, 100);
  tft.print("Rev.1.02");
  tft.setCursor(60, 120);
  tft.print("SM6AFV 2020-04-03");
  tft.setFont(Arial_10);
  delay(2000);
}
//---------------------------------------------------------------
void getMHlocator()
{
  // Test Locators
  //
  // Locator JO67GQ66SX
  //declat = 57.695683 * 1000000.;
  //declong = 12.556446 * 1000000.;
  // Locator JO67GQ57RB
  //declat = 57.696067 * 1000000.;
  //declong = 12.547904 * 1000000.;
  // Locator HK74TW95CS
  //declat = 14.940692 * 1000000.;
  //declong = -24.340631 * 1000000.;
  // Locator BH52IG23PE
  //declat = -17.73674 * 1000000.;
  //declong = -149.311193 * 1000000;

  ////////// Maidenhead Locator calculation ///////////////////////
  // Locator calculation definitions
  char* FirstCharString[] = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X"};
  char* MidCharString[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};
  char* LastCharString[] = {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X"};

  // variable for calculations
  double scrap;
  double rem1, rem2, rem3, rem4, rem5, rem6, rem7, rem8, rem9, rem10; // variable for LOC calculations
  String loc1, loc2, loc3, loc4, loc5, loc6, loc7, loc8, loc9, loc10;
  loclong = longitude * 1000000;
  loclat = latitude * 1000000;

  loclong = loclong + 180000000;
  loclat = loclat + 90000000;
 
  // First Character - longitude based (every 20° = 1 gridsq) //----------
  rem1 = loclong / 1000000 - 20 * int(loclong / 20000000);
  loc1 = FirstCharString[int(loclong / 20000000)];

  // Second Character - latitude based (every 10° = 1 gridsq)
  rem2 = loclat / 1000000 - 10 * int(loclat / 10000000);
  loc2 = FirstCharString[int(loclat / 10000000)];

  // Third Character - longitude based (every 2° = 1 gridsq) //-/10--------
  // scrap calculates the remaining
  rem3 = rem1 - 2 * int(rem1 / 2);
  scrap = loclong  -   (20000000 * int (loclong / 20000000));
  loc3 = MidCharString[int(scrap * 10 / 20 / 1000000)];

  // Fourth Character - latitude based (every 1° = 1 gridsq) //-/10--------
  rem4 = rem2 - 1 * int(rem2 / 1);
  scrap = loclat  -  (10000000 * int (loclat / 10000000));
  loc4 = MidCharString[int(scrap / 1000000)];

  // Fifth Character - longitude based (every 5' = 1 gridsq)   //-/24-------
  rem5 = rem3 - 0.0833333333333333 * int(rem3 / 0.0833333333333333);
  scrap = (loclong / 2000000) - (int (loclong / 2000000));
  loc5 = LastCharString[int(scrap * 24)];

  // Sixth Character - longitude based (every 2.5' = 1 gridsq)  //-/24-----
  rem6 = rem4 - 0.0416666666666667 * int(rem4 / 0.0416666666666667);
  scrap = (loclat / 1000000) - (int (loclat / 1000000));
  loc6 = LastCharString[int(scrap * 24)];

  // Seventh Character - longitude based (every 30" = 1 gridsq)  //-/10-----
  rem7 = rem5 - 0.0083333333333333 * int(rem5 / 0.0083333333333333);
  loc7 = MidCharString[int(rem5 / 0.0083333333333333)];

  // Eights Character - latitude based (every 15" = 1 gridsq)  //-/10------
  rem8 = rem6 - 0.0041666666666667 * int(rem6 / 0.0041666666666667);
  loc8 = MidCharString[int(rem6 / 0.0041666666666667)];

  // Nineth Character - longitude based (every 1.25" = 1 gridsq) //-/24----
  rem9 = rem7 - 0.0003472222222222 * int(rem7 / 0.0003472222222222);
  loc9 = LastCharString[int(rem7 / 0.0003472222222222)];

  // Tenth Character - longitude based (every 0.625" = 1 gridsq) //-/24---
  rem10 = rem8 - 0.0001736111111111 * int(rem8 / 0.0001736111111111);
  loc10 = LastCharString[int(rem8 / 0.0001736111111111)];
  MHlocator = loc1 + loc2 + loc3 + loc4 + loc5 + loc6 + loc7 + loc8 + loc9 + loc10;
}
//-----------------------------------------------------------------------------
//---------------------------------------------------------------
void showAzimuth()
{
  tft.fillScreen(ILI9341_BLACK);
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(0, 10);
  tft.setFont(Arial_10_Bold);
  tft.println("AZIMUTH/LOCATOR DISPLAY:");
  tft.println("-----------------------------------------------");
  tft.setTextColor(ILI9341_WHITE);
  tft.setFont(Arial_12);
  char buffer[15];

  //Date_Time
  tft.print("UTC Date  ");
  tft.print(year);
  if (month <10) tft.print("-0");
  else tft.print("-");
  tft.print(month);
  if (day <10) tft.print("-0");
  else tft.print("-");
  tft.println(day);

  tft.setFont(Arial_14_Bold);
  tft.println();
  tft.print("  UTC Time ");
  if (hour <10) tft.print("0");
  else tft.print(" ");
  tft.print(hour);
  if (minute <10) tft.print(":0");
  else tft.print(":");
  tft.print(minute);
  if (second <10) tft.print(":0");
  else tft.print(":");
  tft.println(second);
  tft.println();
  tft.setFont(Arial_10);

  tft.setFont(Arial_14_Bold);
  tft.print("  Heading (deg):");
  dtostrf(heading, 10, 2, buffer);
  tft.println(buffer);

  tft.setFont(Arial_12);
  tft.println();
  tft.print("relPosLength  (mm):  ");
  dtostrf(relposlength, 10, 2, buffer);
  tft.println(buffer);

  tft.print("AltitudeMSL      (m):");
  dtostrf(altitude, 12, 2, buffer);
  tft.println(buffer);

  tft.setCursor(0, 280);
  tft.setTextColor(ILI9341_YELLOW);
  tft.print("Battery (V):          ");
  tft.println(BatteryVoltage);
  tft.setCursor(0, 300);
  tft.setTextColor(ILI9341_YELLOW);
  tft.print("MHLocator:         ");
  tft.println(MHlocator);
  tft.setTextColor(ILI9341_WHITE);
  tft.setFont(Arial_10);
}
//---------------------------------------------------------------
void getWeather()
{
  tempSHT = sht1x.readTemperatureC();
  humiditySHT = sht1x.readHumidity();
  // Dewpoint Calculation
  double RATIO = 373.15 / (273.15 + tempSHT);
  double SUM = -7.90298 * (RATIO - 1);
  SUM += 5.02808 * log10(RATIO);
  SUM += -1.3816e-7 * (pow(10, (11.344 * (1 - 1 / RATIO ))) - 1) ;
  SUM += 8.1328e-3 * (pow(10, (-3.49149 * (RATIO - 1))) - 1) ;
  SUM += log10(1013.246);
  double VP = pow(10, SUM - 3) * humiditySHT;
  double T = log(VP / 0.61078); // temp var
  dewpointSHT = (241.88 * T) / (17.558 - T);

  //temperature1 = bme.readTemperature();
  //sensors.requestTemperatures();
  //temperature2 = sensors.getTempCByIndex(0);
  //if(temperature2 != DEVICE_DISCONNECTED_C)
  //rhumidity = bme.readHumidity();
  pressure = bme.readPressure()/100.0F;
  //bme.setSampling(Adafruit_BME280::MODE_FORCED,
  //                Adafruit_BME280::SAMPLING_X1,  // temperarture
  //                Adafruit_BME280::SAMPLING_X1,  // pressure
  //                Adafruit_BME280::SAMPLING_X1,  // humidity
  //                Adafruit_BME280::FILTER_OFF );
  //dewpoint1 = DewPoint2(temperature1, rhumidity);
  //dewpoint2 = DewPoint2(temperature2, rhumidity);
}
//---------------------------------------------------------------
void showWeather()
{
  tft.fillScreen(ILI9341_BLACK);
  tft.setTextColor(ILI9341_GREEN);
  tft.setCursor(0, 10);
  tft.setFont(Arial_10_Bold);
  tft.println("WEATHER DISPLAY:");
  tft.println("-----------------------------------------------");
  tft.setTextColor(ILI9341_WHITE);
  tft.setFont(Arial_10);
  char buffer[15];

  //Date_Time
  tft.print("UTC DateTime  ");
  tft.print(year);
  if (month <10) tft.print("-0");
  else tft.print("-");
  tft.print(month);
  if (day <10) tft.print("-0");
  else tft.print("-");
  tft.print(day);
  tft.print(" ");
  if (hour <10) tft.print(":0");
  else tft.print(":");
  tft.print(hour);
   if (minute <10) tft.print(":0");
  else tft.print(":");
  tft.print(minute);
   if (second <10) tft.print(":0");
  else tft.print(":");
  tft.println(second);
  tft.println();

  tft.setFont(Arial_12);
  tft.print("Temperature   (degC): ");
  tft.println(tempSHT);
  tft.print("Rel. Humidity       (%): ");
  tft.println(humiditySHT);
  tft.print("Dewpoint        (degC):   ");
  tft.println(dewpointSHT);
  tft.print("Air Pressure     (hPa): ");
  tft.println(pressure);
  tft.print("Abs Humidity   (g/m3): ");
  tft.println(absHumidity);

  tft.setCursor(0, 300);
  tft.setTextColor(ILI9341_YELLOW);
  tft.print("MHLocator:        ");
  tft.println(MHlocator);
  tft.setTextColor(ILI9341_WHITE);
  tft.setFont(Arial_10);
}
//---------------------------------------------------------------- 
void AbsoluteHumidity()
{
  // tempSHT = Temperature degrees celcius (e.g. 19.5)
  // humiditySHT = Relative Humidity % (e.g. 35)

  // Returns absolute humidity in grams per cubic metre

  // Note: This is an approximation that is valid for temperatures
  // between -20 to +50 degrees. The maximum error is 0.083% of the result.

    absHumidity = 2.16679L * ((6.116441L * pow( 10.,((7.591386L * tempSHT)/(tempSHT + 240.7263L))) * humiditySHT/100.) * 100. /
    (273.15L + tempSHT));  
}
//----------------------------------------------------------------
void getBatteryStatus()
{
    // Test Battery status
    delay(100);
    adVoltage1 = analogRead(batteryPin1);
    adVoltage2 = analogRead(batteryPin2);
    // Read power supply voltage
    //Serial.println(batteryPin1); 
    //Serial.println(batteryPin2);
    Serial.println(adVoltage1); 
    Serial.println(adVoltage2);
    //Serial.println(resistorFactor);
    //Serial.println(referenceVoltage);   
    BatteryVoltage = ((float(adVoltage1) /adcMax) * referenceVoltage * resistorFactor); // Internal ref. voltage & corr. for divider
    Serial.println(BatteryVoltage);  
    if  (BatteryVoltage <= BatteryVoltageMin) // We have a low voltage   
        {                   
          tft.setTextColor(ILI9341_RED);
          tft.setCursor(0,260);
          tft.setFont(Arial_12_Bold);          
          tft.println("LOW BATT!");
          tft.setFont(Arial_12);                
        }    
 } 
 // End BatteryCheck

//----------------------------------------------------------------------------------------------------------------------

